#Load the necessary libraries.
library(gdata)
library(stats)
library(forecast)
library(tseries)
library(fitdistrplus)
library(mixtools)
library(VGAM)
library(moments)

#OutoutFiles
OutputfileMoments <- "MinFil_Moments.txt"
OutputfileCorrelation <- "MinFil_Corr.txt"

#InputFiles 
DA <- read.xls("2013-2016_hourlydata_ibruk.xls", header=FALSE) #NB! Data kun til 22.04.2016 - forecaster for periodsen fra lørdag 23.04 - fredag29.04
PRIM <- read.xls("PrimRes_2013-2016_uten.xls", header=FALSE)

################################ DA PRICES ################################ 
#----------------- Load day ahead data -----------------#
DAts <- ts(DA,start=c(2013,1),frequency=365)

#--------------------- Handle seasonality and trend by determintistic function--------------------- #
#Remove seasonality from all 
RemSeason <- function(DAts) {
  x <- 1:1208
  FSeasonalCompDAts <- lm(DAts ~ x + sin(2*pi/365*x)+cos(2*pi/365*x))
  FSeasonalDAts <- DAts - fitted(FSeasonalCompDAts) 
  return(FSeasonalDAts)
}

RemSeasonTwo <- function(DAts) {
  x <- 1:1208
  FSeasonalCompDAts <- lm(DAts ~ x + sin(2*pi/365*x)+cos(2*pi/365*x))
  Fit <- fitted(FSeasonalCompDAts) 
  return(Fit)
}

RemSeasonThree <- function(DAts) {
  x <- 1:1208
  FSeasonalCompDAts <- lm(DAts ~ x + sin(2*pi/365*x)+cos(2*pi/365*x))
  return(FSeasonalCompDAts)
}

FitBefore <- apply(DAts,2,RemSeasonThree)

Fit <- apply(DAts,2, RemSeasonTwo)
Fitts <- ts(Fit,start=c(2013,1),frequency=365)

SeasonalDA <- apply(DAts,2, RemSeason)
SeasonalDAts <- ts(SeasonalDA,start=c(2013,1),frequency=365)

#----------------- Remove spikes -----------------#
RemoveSpikesOne <- function(DA) {
  l <- length(DA)
  NoSpikesDA <- DA                      #Save the new time series 
  SpikeValue <- matrix(0,nrow=1,ncol=l) #Sample all differences in order to add spikes later
  m <- mean(DA)
  s <- sd(DA)
  for (i in 2:l) {
    positive <- m + 2.5*s
    negative <- m - 2.5*s
    if (NoSpikesDA[i] > positive ) {
      SpikeValue[i] <- NoSpikesDA[i]
      NoSpikesDA[i] <- NoSpikesDA[i-1]
    } else if (NoSpikesDA[i] < negative) {
      SpikeValue[i] <- NoSpikesDA[i]
      NoSpikesDA[i] <- NoSpikesDA[i-1]
    }
  }
  return(SpikeValue)  
}

SpikesDA <- apply(SeasonalDAts,2, RemoveSpikesOne)

RemoveSpikesTwo <- function(DA) {
  l <- length(DA)
  NoSpikesDA <- DA                      #Save the new time series 
  SpikeValue <- matrix(0,nrow=1,ncol=l) #Sample all differences in order to add spikes later
  m <- mean(DA)
  s <- sd(DA)
  for (i in 2:l) {
    positive <- m + 2.5*s
    negative <- m - 2.5*s
    if (NoSpikesDA[i] > positive ) {
      SpikeValue[i] <- NoSpikesDA[i]
      NoSpikesDA[i] <- NoSpikesDA[i-1]
    } else if (NoSpikesDA[i] < negative) {
      SpikeValue[i] <- NoSpikesDA[i]
      NoSpikesDA[i] <- NoSpikesDA[i-1]
    }
  } #NB! Endre her! 
  return(NoSpikesDA) #NB! Endre her! 
}

NoSpikes <- apply(SeasonalDAts,2, RemoveSpikesTwo)
NoSpikests <- ts(NoSpikes,start=c(2013,1),frequency=365)

#--------------------- Add seasoanlity  --------------------- #
OriginalSeriesWithoutSpikes <- NoSpikests + Fitts

#--------------------- Variance stabilizing - log-transformation --------------------- #
BXOne <- log(OriginalSeriesWithoutSpikes[,1])
BXTwo <- log(OriginalSeriesWithoutSpikes[,2])
BXThree <- log(OriginalSeriesWithoutSpikes[,3])
BXFour <- log(OriginalSeriesWithoutSpikes[,4])
BXFive <- log(OriginalSeriesWithoutSpikes[,5])
BXSix <- log(OriginalSeriesWithoutSpikes[,6])
BXSeven <- log(OriginalSeriesWithoutSpikes[,7])
BXEight <- log(OriginalSeriesWithoutSpikes[,8])
BXNine <- log(OriginalSeriesWithoutSpikes[,9])
BXTen <- log(OriginalSeriesWithoutSpikes[,10])
BXElleven <- log(OriginalSeriesWithoutSpikes[,11])
BXTwelve <- log(OriginalSeriesWithoutSpikes[,12])
BXThirteen <- log(OriginalSeriesWithoutSpikes[,13])
BXFourteen <- log(OriginalSeriesWithoutSpikes[,14])
BXFifteen <- log(OriginalSeriesWithoutSpikes[,15])
BXSixteen <- log(OriginalSeriesWithoutSpikes[,16])
BXSeventeen <- log(OriginalSeriesWithoutSpikes[,17])
BXEighteen <- log(OriginalSeriesWithoutSpikes[,18])
BXNineteen <- log(OriginalSeriesWithoutSpikes[,19])
BXTwenty <- log(OriginalSeriesWithoutSpikes[,20])
BXTwentyOne <- log(OriginalSeriesWithoutSpikes[,21])
BXTwentyTwo <- log(OriginalSeriesWithoutSpikes[,22])
BXTwentyThree <- log(OriginalSeriesWithoutSpikes[,23])
BXTwentyFour <- log(OriginalSeriesWithoutSpikes[,24])

BXtransformedSeriesOne <- cbind(BXOne,BXTwo,BXThree,BXFour,BXFive,BXSix,BXSeven,BXEight,BXNine,BXTen,BXElleven,BXTwelve)
BXtransformedSeriesTwo <- cbind(BXThirteen,BXFourteen,BXFifteen,BXSixteen,BXSeventeen,BXEighteen,BXNineteen,BXTwenty,BXTwentyOne,BXTwentyTwo,BXTwentyThree,BXTwentyFour)
BXtransformedSeries <- cbind(BXtransformedSeriesOne, BXtransformedSeriesTwo)

#--------------------- Remove seasonality again --------------------- #
#Remove seasonality from all terms 
RemSeason <- function(DAts) {
  x <- 1:1208
  FSeasonalCompDAts <- lm(DAts ~ x + sin(2*pi/365*x)+cos(2*pi/365*x))
  FSeasonalDAts <- DAts - fitted(FSeasonalCompDAts) 
  Fit <- fitted(FSeasonalCompDAts)
  return(FSeasonalDAts)
}

RemSeasonTwo <- function(DAts) {
  x <- 1:1208
  FSeasonalCompDAts <- lm(DAts ~ x + sin(2*pi/365*x)+cos(2*pi/365*x))
  FSeasonalDAts <- DAts - fitted(FSeasonalCompDAts) 
  Fit <- fitted(FSeasonalCompDAts)
  return(FSeasonalCompDAts)
}

RemSeasonThree <- function(DAts) {
  x <- 1:1208
  FSeasonalCompDAts <- lm(DAts ~ x + sin(2*pi/365*x)+cos(2*pi/365*x))
  Fit <- fitted(FSeasonalCompDAts)
  return(Fit)
}

FitAfterDA <- apply(BXtransformedSeries, 2, RemSeasonTwo)
FitAfterDAts <- ts(FitAfterDA,start=c(2013,1),frequency=365)

FitBX <- apply(BXtransformedSeries, 2, RemSeasonThree)
FitBXts <- ts(FitBX,start=c(2013,1),frequency=365)

SeasonalAfterDA <- apply(BXtransformedSeries,2, RemSeason)
SeasonalAfterDAts <- ts(SeasonalAfterDA,start=c(2013,1),frequency=365)

#------------------------ Test for stationarity  ------------------------ #
Atest <- function(SDAts) {
  adf.test(SDAts)
}

AugDFtest <- apply(SeasonalAfterDAts,2,Atest)

#---------------- Fit ARMA model to the hourly prices & extract residuals ---------------#
RunArima <- function(DAts) {
  ArimaFitDAts <- auto.arima(DAts,,max.d=0, max.p=3,max.q=3, seasonal=FALSE)
  DAsummary <- summary(ArimaFitDAts)
  DAresiduals <- ArimaFitDAts$residuals
  return(list(DAarima=ArimaFitDAts, S = DAsummary, Res=DAresiduals))
}

DA.l <- apply(SeasonalAfterDAts,2, RunArima)

#---------------- Put residuals in matrix ---------------#
DAResidualsMatrixOne <- cbind(DA.l$BXtransformedSeriesOne.BXOne$Res, DA.l$BXtransformedSeriesOne.BXTwo$Res, DA.l$BXtransformedSeriesOne.BXThree$Res,DA.l$BXtransformedSeriesOne.BXFour$Res,DA.l$BXtransformedSeriesOne.BXFive$Res,DA.l$BXtransformedSeriesOne.BXSix$Res,DA.l$BXtransformedSeriesOne.BXSeven$Res,DA.l$BXtransformedSeriesOne.BXEight$Res,DA.l$BXtransformedSeriesOne.BXNine$Res,DA.l$BXtransformedSeriesOne.BXTen$Res,DA.l$BXtransformedSeriesOne.BXElleven$Res,DA.l$BXtransformedSeriesOne.BXTwelve$Res)
DAResidualsMatrixTwo <- cbind(DA.l$BXtransformedSeriesTwo.BXThirteen$Res, DA.l$BXtransformedSeriesTwo.BXFourteen$Res, DA.l$BXtransformedSeriesTwo.BXFifteen$Res,DA.l$BXtransformedSeriesTwo.BXSixteen$Res,DA.l$BXtransformedSeriesTwo.BXSeventeen$Res,DA.l$BXtransformedSeriesTwo.BXEighteen$Res,DA.l$BXtransformedSeriesTwo.BXNineteen$Res,DA.l$BXtransformedSeriesTwo.BXTwenty$Res,DA.l$BXtransformedSeriesTwo.BXTwentyOne$Res,DA.l$BXtransformedSeriesTwo.BXTwentyTwo$Res,DA.l$BXtransformedSeriesTwo.BXTwentyThree$Res,DA.l$BXtransformedSeriesTwo.BXTwentyFour$Res)
DAResidualsMatrix <- cbind(DAResidualsMatrixOne,DAResidualsMatrixTwo)

#------------- BUILT IN functions to be used to find the PCA components (NB! Cor=FALSE - Uses covaraince matrix) -------------#
PCADayAhead <- princomp(DAResidualsMatrix, scores=TRUE, center=FALSE, cor=FALSE) #scores - transform data! Benytter corelation matric når cor er satt til TRUE. 
summary(PCADayAhead)

#------------- MANUAL PCA analysis to find the matrixes and conduct the transformations -------------#
#Set mean value to zero 
cmDA=colMeans(DAResidualsMatrix)
DayAheadDataPCA = DAResidualsMatrix #- matrix(cmDA,nrow=dim(DAResidualsMatrix)[1], ncol=length(cmDA),byrow=TRUE)

#Find covariance  (NB! Could also have chosen to use the correlation matrix)
DayAheadDataCov <- cov(DayAheadDataPCA)
DayAheadDataEigen <- eigen(DayAheadDataCov)
DayAheadDataEigenVectors <- DayAheadDataEigen$vectors

#find P = XW (X= historical data, W= eigenvectors)
DayAheadDataPCAMatrix <- as.matrix(DayAheadDataPCA)
DayAheadDataNew <- DayAheadDataPCAMatrix %*% DayAheadDataEigenVectors

#Choose number of PCA terms 
PCAtermsDA = 10

#Chosen set of resulting vectors  
DayAheadDataNewCorrect <- DayAheadDataNew[,1:PCAtermsDA]

#------------ EXTRACT MOMENTS OF THE MAIN Principal components ------------#
MomentsDA <- matrix(0,nrow = 4, ncol = PCAtermsDA) #Ncol is the same as the number of residuals 
TransposedMatrixDA <- t(DayAheadDataNewCorrect)

i=1
for (j in 1:PCAtermsDA) {
  MomentsDA[1,i] <- mean(TransposedMatrixDA[j,])
  MomentsDA[2,i] <- sd(TransposedMatrixDA[j,])
  MomentsDA[3,i] <- skewness(TransposedMatrixDA[j,])
  MomentsDA[4,i] <- kurtosis(TransposedMatrixDA[j,])
  i=i+1
}


############# PRIMARY RESERVES #############
#----------------- Load weekly Primary reserve data -----------------#
PRIMts <- ts(PRIM[,1:6], start=c(2013,1),frequency=52)

#----------------- Load average blockwise DA values  -----------------#
DAblock <- read.xls("DA_averageblock.xls", header=FALSE)
DAblockts <- ts(DAblock,start=c(2013,1),frequency=52)
summary(DAblockts)

cor(PRIM,DAblock)

#------------------------ Preprocess the data (do a Box-Cox transformation) ------------------------ #
PrimWN <- log(PRIM[,1]+1)
PrimWD <- log(PRIM[,2]+1)
PrimWE <- log(PRIM[,3]+1)
PrimWeekendN <- log(PRIM[,4]+1)
PrimWeekendD <- log(PRIM[,5]+1)
PrimWeekendE <- log(PRIM[,6]+1)


#---------------- Fit ARMAX model to the 6 block prices & extract error terms ---------------#
AprimWNts <- auto.arima(PrimWN,max.d=0, max.p=3,max.q=3, xreg=log(DAblock[,1]))
summary(AprimWNts)
ResAprimWNts <- AprimWNts$residuals

AprimWDts <- auto.arima(PrimWD,max.d=0, max.p=3,max.q=3,xreg=log(DAblock[,2]))
summary(AprimWDts)
ResAprimWDts <- AprimWDts$residuals

AprimWEts <- auto.arima(PrimWE,max.d=0, max.p=3,max.q=3, xreg=log(DAblock[,3]))
summary(AprimWEts)
ResAprimWEts <- AprimWEts$residuals

AprimWeekendNts <- auto.arima(PrimWeekendN,,max.d=0, max.p=3,max.q=3, xreg=log(DAblock[,4]))
summary(AprimWeekendNts)
ResAprimWeekendNts <- AprimWeekendNts$residuals

AprimWeekendDts <- auto.arima(PrimWeekendD,max.d=0, max.p=3,max.q=3, xreg=log(DAblock[,5]))
summary(AprimWeekendDts)
ResAprimWeekendDts <- AprimWeekendDts$residuals

AprimWeekendEts <- auto.arima(PrimWeekendE,max.d=0, max.p=3,max.q=3, xreg=log(DAblock[,6]))
summary(AprimWeekendEts)
ResAprimWeekendEts <- AprimWeekendEts$residuals

#------------- BUILT IN functions to be used to find the PCA components (NB! Cor=FALSE - Uses covaraince matrix) -------------#
PrimData <- cbind(ResAprimWNts,ResAprimWDts , ResAprimWEts ,ResAprimWeekendNts, ResAprimWeekendDts, ResAprimWeekendEts)

AugDFtest <- apply(PrimData,2,Atest)

PCAPrimData <- princomp(PrimData, scores=TRUE, center=FALSE, cor=FALSE) #scores - transform data! Benytter corelation matric når cor er satt til TRUE. 
summary(PCAPrimData)

#------------- MANUAL PCA analysis to find the matrixes and do the transformations -------------#
#Set mean value to zero 
cmPrim=colMeans(PrimData)
PrimDataPCA = PrimData - matrix(cmPrim,nrow=dim(PrimData)[1], ncol=length(cmPrim),byrow=TRUE)
PrimDataPCAMatrix <- as.matrix(PrimDataPCA)

#find covariance  (NB! Could also have chosen to use the correlation matrix)
PrimDataCov <- cov(PrimDataPCA)
PrimDataEigen <- eigen(PrimDataCov)
PrimDataEigenVectors <- PrimDataEigen$vectors

#find P = XW (X= historical data, W= eigenvectors)
PrimDataNew <- PrimDataPCAMatrix %*% PrimDataEigenVectors

#Choose number of PCA terms 
PCAtermsPrim = 3

#Chosen set of eigenvectors 
PrimDataNEWCorrect <- PrimDataNew[,1:PCAtermsPrim] 

#------------ EXTRACT MOMENTS OF THE MAIN Principal components ------------#
MomentsPrim <- matrix(0,nrow = 4, ncol = PCAtermsPrim) #Ncol is the same as the number of residuals 
TransposedMatrixPrim <- t(PrimDataNEWCorrect)

for (j in 1:PCAtermsPrim) {
  MomentsPrim[1,j] <- mean(TransposedMatrixPrim[j,])
  MomentsPrim[2,j] <- sd(TransposedMatrixPrim[j,])
  MomentsPrim[3,j] <- skewness(TransposedMatrixPrim[j,])
  MomentsPrim[4,j] <- kurtosis(TransposedMatrixPrim[j,])
}


###### WRITE DA and PRIM reserve moments and correlation to file #########
CorrelationsBoth <- c(1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
                      0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
                      0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
                      0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0,
                      0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0,
                      0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0,
                      0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0,
                      0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0,
                      0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0,
                      0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0,
                      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0,
                      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0,
                      0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1)

MomentsBoth <- cbind(MomentsDA,MomentsPrim)

# -------------------- Write to file --------------------#
x <- c(4, PCAtermsDA + PCAtermsPrim, " ") 
write(x, file = OutputfileMoments, ncolumns = 1, append = FALSE, sep = " ")
write(t(MomentsBoth), file = OutputfileMoments, ncolumns = PCAtermsDA+PCAtermsPrim, append = TRUE, sep = " ") #Rows = moment, Col=PCA

y <- c(PCAtermsDA + PCAtermsPrim, PCAtermsDA + PCAtermsPrim, " ")
write(y, file = OutputfileCorrelation, ncolumns = 1, append = FALSE, sep = " ")
write(CorrelationsBoth, file = OutputfileCorrelation, ncolumns = PCAtermsDA+PCAtermsPrim, append = TRUE, sep = " ") #Note that rows are the same moment, columns are for one PCa


