#Load the necessary libraries..
library(gdata)
library(stats)
library(forecast)
library(tseries)
library(fitdistrplus)
library(mixtools)
library(VGAM)
library(moments)

InputDataFromC <- "InputFileFromMomentMatchingAlgorithm.txt"

#--------------------- INPUT from scenariogeneration outputfile in C++ ---------------------#
OutputScenarios <- read.table(InputDataFromC)
ScenariosDA <- nrow(OutputScenarios) #Scenarios for both DA and PrimRes
nScenarios <- nrow(OutputScenarios)

From <- PCAtermsDA+2
To <- From + PCAtermsPrim -1 
PCAScenarioPrim <- OutputScenarios[,From:To]
PCAScenarioMatrixPrim <- as.matrix(PCAScenarioPrim)

PCAScenariosDAto <- OutputScenarios[,2:(PCAtermsDA+1)]
PCAScenariosMatrixDA <- as.matrix(PCAScenariosDAto)


########## PRIMARY RESERVES ############
#--------------------- TRANSFORM BACK TO ORIGINAL VALUES X = P'*W´^T (NB! error-terms) ---------------------#
OrigPCAScenariosWithoutMeanPrim1 <-    PCAScenarioMatrixPrim %*% t(PrimDataEigenVectors[,1:PCAtermsPrim])
OriginalPCAScenariosPrim <- OrigPCAScenariosWithoutMeanPrim1  + matrix(cmPrim,nrow=ScenariosDA, ncol=length(cmPrim),byrow=TRUE)

#--------------------- FORECAST FOR NEXT WEEK with ARMAX ---------------------#
ResForecast <- function(PrimError, Modell, WeeklyBlockDAvalue) {
  PrimFor <- simulate(Modell, nsim=1, innov=PrimError, xreg=log(WeeklyBlockDAvalue), future=TRUE)
  return(PrimFor)
} 

#Generate forecast for all 6 blocks 
PrimScenarioWN <- apply(OriginalPCAScenariosPrim[,1, drop=F],1, ResForecast, Modell=AprimWNts, AverageValueInBlock[1,1])
PrimScenarioWNBX <- exp(PrimScenarioWN)-1
PrimScenarioWD <- apply(OriginalPCAScenariosPrim[,2, drop=F],1, ResForecast, Modell=AprimWDts, AverageValueInBlock[1,2])
PrimScenarioWDBX<- exp(PrimScenarioWD)-1
PrimScenarioWE <- apply(OriginalPCAScenariosPrim[,3, drop=F],1, ResForecast, Modell=AprimWEts, AverageValueInBlock[1,3])
PrimScenarioWEBX <- exp(PrimScenarioWE)-1
PrimScenarioWeekendN <- apply(OriginalPCAScenariosPrim[,4, drop=F],1, ResForecast, Modell=AprimWeekendNts, AverageValueInBlock[1,4])
PrimScenarioWeekendNBX <- exp(PrimScenarioWeekendN)-1
PrimScenarioWeekendD <- apply(OriginalPCAScenariosPrim[,5, drop=F],1, ResForecast, Modell=AprimWeekendDts, AverageValueInBlock[1,5])
PrimScenarioWeekendDBX <- exp(PrimScenarioWeekendD)-1
PrimScenarioWeekendE <- apply(OriginalPCAScenariosPrim[,6, drop=F],1, ResForecast, Modell=AprimWeekendEts, AverageValueInBlock[1,6])
PrimScenarioWeekendEBX <- exp(PrimScenarioWeekendE)-1

PrimScenarios <- cbind(PrimScenarioWNBX,PrimScenarioWDBX,PrimScenarioWEBX,PrimScenarioWeekendNBX,PrimScenarioWeekendDBX,PrimScenarioWeekendEBX)

#Find historical max and min price (for primary reserves - throughout the year)
PrimHistoricalMaxPrice <- matrix(0,nrow = 1, ncol = 6) #Ncol is the same as the number of residuals 
PrimHistoricalMinPrice <- matrix(0,nrow = 1, ncol = 6) 

for (i in 1:6) {
  PrimHistoricalMaxPrice[1,i] <- max(PRIM[,i])
  PrimHistoricalMinPrice[1,i] <- min(PRIM[,i])
}

#Remove scenarios with max and min price above or below the historical max and min prices & replace minus values with 0 
CorrectScenariosPre <- matrix(0, nrow=1,ncol=6)
DAScenariosResiduals <- matrix(0,nrow=1,ncol=PCAtermsDA)
k=0
SelectedScenarios=0
for (i in 1:nScenarios) {
    if (PrimScenarios[i,1] < PrimHistoricalMaxPrice[1,1] && PrimScenarios[i,2] < PrimHistoricalMaxPrice[1,2] && PrimScenarios[i,3] < PrimHistoricalMaxPrice[1,3] && PrimScenarios[i,4] < PrimHistoricalMaxPrice[1,4] && PrimScenarios[i,5] < PrimHistoricalMaxPrice[1,5] && PrimScenarios[i,6] < PrimHistoricalMaxPrice[1,6]) {
      CorrectScenariosPre <- rbind(CorrectScenariosPre,PrimScenarios[i,])
      DAScenariosResiduals <- rbind(DAScenariosResiduals,PCAScenariosMatrixDA[i,] )
      k=k+1
      SelectedScenarios <- SelectedScenarios + 1
    }
      if (CorrectScenariosPre[k,1] < 0.01) {
        CorrectScenariosPre[k,1] <- 0
      }
      if (CorrectScenariosPre[k,2] < 0.01) {
        CorrectScenariosPre[k,2] <- 0
      }
      if (CorrectScenariosPre[k,3] < 0.01) {
        CorrectScenariosPre[k,3] <- 0
      }
      if (CorrectScenariosPre[k,4] < 0.01) {
        CorrectScenariosPre[k,4] <- 0
      }
      if (CorrectScenariosPre[k,5] < 0.01) {
        CorrectScenariosPre[k,5] <- 0
      }
      if (CorrectScenariosPre[k,6] < 0.01) {
        CorrectScenariosPre[k,6] <- 0
      }
}

PrimCorrectScenarios <- CorrectScenariosPre[2:nrow(CorrectScenariosPre),]
DACorrextScenariosResiduals <- DAScenariosResiduals[2:nrow(CorrectScenariosPre),]


################# DA VALUES ############################
OriginalScenariosDA <- DACorrextScenariosResiduals  %*% t(DayAheadDataEigenVectors[,1:PCAtermsDA])

#--------------- the block of 24 hours for each day ---------------------# NB! Sampler kun fra scenarier som er utvalgt fra PRIM
Values <- c(1:SelectedScenarios)
RandomErrorsInWeekDA <- matrix(0,nrow = SelectedScenarios, ncol = 7) 
RandomErrorsInWeekDA[,1] <- c(1:SelectedScenarios)

for (i in 2:7) {
  RandomErrorsInWeekDA[,i] <- sample(Values, SelectedScenarios, replace=FALSE)
}

WeeklyErrors <- function(OriginalScenariosVectorDA) {
  ErrorMatrixDA <- matrix(0,nrow = SelectedScenarios, ncol = 7) 
  for (i in 1:SelectedScenarios) { 
    for (j in 1:7) { 
      V <- RandomErrorsInWeekDA[i,j]
      ErrorMatrixDA[i,j] <- OriginalScenariosVectorDA[V]
    }
  }
  return(ErrorMatrixDA)
}

OriginalScenariosHourOneDA <- WeeklyErrors(OriginalScenariosDA[,1])
OriginalScenariosHourTwoDA <- WeeklyErrors(OriginalScenariosDA[,2])
OriginalScenariosHourThreeDA <- WeeklyErrors(OriginalScenariosDA[,3])
OriginalScenariosHourFourDA <- WeeklyErrors(OriginalScenariosDA[,4])
OriginalScenariosHourFiveDA <- WeeklyErrors(OriginalScenariosDA[,5])
OriginalScenariosHourSixDA <- WeeklyErrors(OriginalScenariosDA[,6])
OriginalScenariosHourSevenDA <- WeeklyErrors(OriginalScenariosDA[,7])
OriginalScenariosHourEightDA <- WeeklyErrors(OriginalScenariosDA[,8])
OriginalScenariosHourNineDA <- WeeklyErrors(OriginalScenariosDA[,9])
OriginalScenariosHourTenDA <- WeeklyErrors(OriginalScenariosDA[,10])
OriginalScenariosHourEllevenDA <- WeeklyErrors(OriginalScenariosDA[,11])
OriginalScenariosHourTwelveDA <- WeeklyErrors(OriginalScenariosDA[,12])
OriginalScenariosHourThirteenDA <- WeeklyErrors(OriginalScenariosDA[,13])
OriginalScenariosHourFourteenDA <- WeeklyErrors(OriginalScenariosDA[,14])
OriginalScenariosHourFifteenDA <- WeeklyErrors(OriginalScenariosDA[,15])
OriginalScenariosHourSixteenDA <- WeeklyErrors(OriginalScenariosDA[,16])
OriginalScenariosHourSeventeenDA <- WeeklyErrors(OriginalScenariosDA[,17])
OriginalScenariosHourEighteenDA <- WeeklyErrors(OriginalScenariosDA[,18])
OriginalScenariosHourNineteenDA <- WeeklyErrors(OriginalScenariosDA[,19])
OriginalScenariosHourTwentyDA <- WeeklyErrors(OriginalScenariosDA[,20])
OriginalScenariosHourTwentyOneDA <- WeeklyErrors(OriginalScenariosDA[,21])
OriginalScenariosHourTwentyTwoDA <- WeeklyErrors(OriginalScenariosDA[,22])
OriginalScenariosHourTwentyThreeDA <- WeeklyErrors(OriginalScenariosDA[,23])
OriginalScenariosHourTwentyFourDA <- WeeklyErrors(OriginalScenariosDA[,24])

#--------------------- FORECAST FOR NEXT DAYS WITH ARIMA MODEL (and add seasonality+do inverse boxcox transformation) ---------------------#
ForecastDA <- function(PrimError, Modell) {
  DAFor <- simulate(Modell, nsim=7, innov=PrimError, future=TRUE)
  return(DAFor)
}  

DAScenarioHourOne <- apply(OriginalScenariosHourOneDA,1,ForecastDA, Modell=DA.l$BXtransformedSeriesOne.BXOne$DAarima) + predict(FitAfterDA$BXtransformedSeriesOne.BXOne,data.frame(x=c(1209:1215)))
DAScenarioHourOneBX <- exp(DAScenarioHourOne) - predict(FitBefore$V1,data.frame(x=c(1209:1215)))
DAScenarioHourTwo <- apply(OriginalScenariosHourTwoDA,1, ForecastDA, Modell=DA.l$BXtransformedSeriesOne.BXTwo$DAarima) + predict(FitAfterDA$BXtransformedSeriesOne.BXTwo,data.frame(x=c(1209:1215)))
DAScenarioHourTwoBX <-exp(DAScenarioHourTwo) - predict(FitBefore$V2,data.frame(x=c(1209:1215)))
DAScenarioHourThree <- apply(OriginalScenariosHourThreeDA,1,ForecastDA, Modell=DA.l$BXtransformedSeriesOne.BXThree$DAarima) + predict(FitAfterDA$BXtransformedSeriesOne.BXThree,data.frame(x=c(1209:1215)))
DAScenarioHourThreeBX <- exp(DAScenarioHourThree) - predict(FitBefore$V3,data.frame(x=c(1209:1215)))
DAScenarioHourFour <- apply(OriginalScenariosHourFourDA,1,ForecastDA, Modell=DA.l$BXtransformedSeriesOne.BXFour$DAarima) + predict(FitAfterDA$BXtransformedSeriesOne.BXFour,data.frame(x=c(1209:1215)))
DAScenarioHourFourBX <- exp(DAScenarioHourFour) - predict(FitBefore$V4,data.frame(x=c(1209:1215)))
DAScenarioHourFive <- apply(OriginalScenariosHourFiveDA,1,ForecastDA, Modell=DA.l$BXtransformedSeriesOne.BXFive$DAarima) + predict(FitAfterDA$BXtransformedSeriesOne.BXFive,data.frame(x=c(1209:1215)))
DAScenarioHourFiveBX <- exp(DAScenarioHourFive) - predict(FitBefore$V5,data.frame(x=c(1209:1215)))
DAScenarioHourSix <- apply(OriginalScenariosHourSixDA,1,ForecastDA, Modell=DA.l$BXtransformedSeriesOne.BXSix$DAarima) + predict(FitAfterDA$BXtransformedSeriesOne.BXSix,data.frame(x=c(1209:1215)))
DAScenarioHourSixBX <- exp(DAScenarioHourSix) - predict(FitBefore$V6,data.frame(x=c(1209:1215)))
DAScenarioHourSeven <- apply(OriginalScenariosHourSevenDA,1,ForecastDA, Modell=DA.l$BXtransformedSeriesOne.BXSeven$DAarima) + predict(FitAfterDA$BXtransformedSeriesOne.BXSeven,data.frame(x=c(1209:1215)))
DAScenarioHourSevenBX <- exp(DAScenarioHourSeven) - predict(FitBefore$V7,data.frame(x=c(1209:1215)))
DAScenarioHourEight <- apply(OriginalScenariosHourEightDA,1,ForecastDA, Modell=DA.l$BXtransformedSeriesOne.BXEight$DAarima) + predict(FitAfterDA$BXtransformedSeriesOne.BXEight,data.frame(x=c(1209:1215)))
DAScenarioHourEightBX <- exp(DAScenarioHourEight) - predict(FitBefore$V8,data.frame(x=c(1209:1215)))
DAScenarioHourNine <- apply(OriginalScenariosHourNineDA,1,ForecastDA, Modell=DA.l$BXtransformedSeriesOne.BXNine$DAarima) + predict(FitAfterDA$BXtransformedSeriesOne.BXNine,data.frame(x=c(1209:1215)))
DAScenarioHourNineBX <- exp(DAScenarioHourNine) - predict(FitBefore$V9,data.frame(x=c(1209:1215)))
DAScenarioHourTen <- apply(OriginalScenariosHourTenDA,1,ForecastDA, Modell=DA.l$BXtransformedSeriesOne.BXTen$DAarima) + predict(FitAfterDA$BXtransformedSeriesOne.BXTen,data.frame(x=c(1209:1215)))
DAScenarioHourTenBX <- exp(DAScenarioHourTen) - predict(FitBefore$V10,data.frame(x=c(1209:1215)))
DAScenarioHourElleven <- apply(OriginalScenariosHourEllevenDA,1,ForecastDA, Modell=DA.l$BXtransformedSeriesOne.BXElleven$DAarima) + predict(FitAfterDA$BXtransformedSeriesOne.BXElleven,data.frame(x=c(1209:1215)))
DAScenarioHourEllevenBX <- exp(DAScenarioHourElleven) - predict(FitBefore$V11,data.frame(x=c(1209:1215))) 
DAScenarioHourTwelve <- apply(OriginalScenariosHourTwelveDA,1,ForecastDA, Modell=DA.l$BXtransformedSeriesOne.BXTwelve$DAarima) + predict(FitAfterDA$BXtransformedSeriesOne.BXTwelve,data.frame(x=c(1209:1215)))
DAScenarioHourTwelveBX <- exp(DAScenarioHourTwelve) - predict(FitBefore$V12,data.frame(x=c(1209:1215)))
DAScenarioHourThirteen <- apply(OriginalScenariosHourThirteenDA,1,ForecastDA, Modell=DA.l$BXtransformedSeriesTwo.BXThirteen$DAarima) + predict(FitAfterDA$BXtransformedSeriesTwo.BXThirteen,data.frame(x=c(1209:1215)))
DAScenarioHourThirteenBX <- exp(DAScenarioHourThirteen) - predict(FitBefore$V13,data.frame(x=c(1209:1215)))
DAScenarioHourFourteen <- apply(OriginalScenariosHourFourteenDA,1,ForecastDA, Modell=DA.l$BXtransformedSeriesTwo.BXFourteen$DAarima) + predict(FitAfterDA$BXtransformedSeriesTwo.BXFourteen,data.frame(x=c(1209:1215)))
DAScenarioHourFourteenBX <- exp(DAScenarioHourFourteen) - predict(FitBefore$V14,data.frame(x=c(1209:1215)))
DAScenarioHourFifteen <- apply(OriginalScenariosHourFifteenDA,1,ForecastDA, Modell=DA.l$BXtransformedSeriesTwo.BXFifteen$DAarima) + predict(FitAfterDA$BXtransformedSeriesTwo.BXFifteen,data.frame(x=c(1209:1215)))
DAScenarioHourFifteenBX <- exp(DAScenarioHourFifteen) - predict(FitBefore$V15,data.frame(x=c(1209:1215)))
DAScenarioHourSixteen <- apply(OriginalScenariosHourSixteenDA,1,ForecastDA, Modell=DA.l$BXtransformedSeriesTwo.BXSixteen$DAarima) + predict(FitAfterDA$BXtransformedSeriesTwo.BXSixteen,data.frame(x=c(1209:1215)))
DAScenarioHourSixteenBX <- exp(DAScenarioHourSixteen) - predict(FitBefore$V16,data.frame(x=c(1209:1215)))
DAScenarioHourSeventeen <- apply(OriginalScenariosHourSeventeenDA,1,ForecastDA, Modell=DA.l$BXtransformedSeriesTwo.BXSeventeen$DAarima) + predict(FitAfterDA$BXtransformedSeriesTwo.BXSeventeen,data.frame(x=c(1209:1215)))
DAScenarioHourSeventeenBX <- exp(DAScenarioHourSeventeen) - predict(FitBefore$V17,data.frame(x=c(1209:1215)))
DAScenarioHourEighteen <- apply(OriginalScenariosHourEighteenDA,1,ForecastDA, Modell=DA.l$BXtransformedSeriesTwo.BXEighteen$DAarima) + predict(FitAfterDA$BXtransformedSeriesTwo.BXEighteen,data.frame(x=c(1209:1215)))
DAScenarioHourEighteenBX <- exp(DAScenarioHourEighteen) - predict(FitBefore$V18,data.frame(x=c(1209:1215)))
DAScenarioHourNineteen <- apply(OriginalScenariosHourNineteenDA,1,ForecastDA, Modell=DA.l$BXtransformedSeriesTwo.BXNineteen$DAarima) + predict(FitAfterDA$BXtransformedSeriesTwo.BXNineteen,data.frame(x=c(1209:1215)))
DAScenarioHourNineteenBX <- exp(DAScenarioHourNineteen) - predict(FitBefore$V19,data.frame(x=c(1209:1215)))
DAScenarioHourTwenty <- apply(OriginalScenariosHourTwentyDA,1,ForecastDA, Modell=DA.l$BXtransformedSeriesTwo.BXTwenty$DAarima) + predict(FitAfterDA$BXtransformedSeriesTwo.BXTwenty,data.frame(x=c(1209:1215)))
DAScenarioHourTwentyBX <- exp(DAScenarioHourTwenty) - predict(FitBefore$V20,data.frame(x=c(1209:1215)))
DAScenarioHourTwentyOne <- apply(OriginalScenariosHourTwentyOneDA,1,ForecastDA, Modell=DA.l$BXtransformedSeriesTwo.BXTwentyOne$DAarima) + predict(FitAfterDA$BXtransformedSeriesTwo.BXTwentyOne,data.frame(x=c(1209:1215)))
DAScenarioHourTwentyOneBX <- exp(DAScenarioHourTwentyOne) - predict(FitBefore$V21,data.frame(x=c(1209:1215)))
DAScenarioHourTwentyTwo <- apply(OriginalScenariosHourTwentyTwoDA,1,ForecastDA, Modell=DA.l$BXtransformedSeriesTwo.BXTwentyTwo$DAarima) + predict(FitAfterDA$BXtransformedSeriesTwo.BXTwentyTwo,data.frame(x=c(1209:1215)))
DAScenarioHourTwentyTwoBX <- exp(DAScenarioHourTwentyTwo) - predict(FitBefore$V22,data.frame(x=c(1209:1215)))
DAScenarioHourTwentyThree <- apply(OriginalScenariosHourTwentyThreeDA,1,ForecastDA, Modell=DA.l$BXtransformedSeriesTwo.BXTwentyThree$DAarima) + predict(FitAfterDA$BXtransformedSeriesTwo.BXTwentyThree,data.frame(x=c(1209:1215)))
DAScenarioHourTwentyThreeBX <- exp(DAScenarioHourTwentyThree) - predict(FitBefore$V23,data.frame(x=c(1209:1215)))
DAScenarioHourTwentyFour <- apply(OriginalScenariosHourTwentyFourDA,1,ForecastDA, Modell=DA.l$BXtransformedSeriesTwo.BXTwentyFour$DAarima) + predict(FitAfterDA$BXtransformedSeriesTwo.BXTwentyFour,data.frame(x=c(1209:1215)))
DAScenarioHourTwentyFourBX <- exp(DAScenarioHourTwentyFour) - predict(FitBefore$V24,data.frame(x=c(1209:1215)))


#--------------------- Add spikes ---------------------#
#Which spikes to sample from (Choose manually - only negative spike sampling implemented)
SpikesInPeriod <- matrix(0,nrow=90,ncol=24)
SpikesInPeriod[1:30,] <- SpikesDA[121:150,]
SpikesInPeriod[31:60,] <- SpikesDA[486:515,]
SpikesInPeriod[61:90,] <- SpikesDA[851:880,]

#Extract only the spikes 
SpikeValueVectorNegative <- SpikesInPeriod[SpikesInPeriod < 0]
SpikeValueVectorPositive <- SpikesInPeriod[SpikesInPeriod > 0]

#Get two matrixes - one for spikes given no spike in the hour before and one given spike before 
SpikesConditionalOnSpike <- matrix(0, nrow=1,ncol=24)
NoSpikesConditionalOnSpike <- matrix(0, nrow=1,ncol=24)
SpikesConditionalOnNoSpike <- matrix(0, nrow=1,ncol=24)
NoSpikesConditionalOnNoSpike <- matrix(0, nrow=1,ncol=24) 

SpikesConditionalOnSpikeProb <- matrix(0, nrow=1,ncol=24)
SpikesConditionalOnNoSpikeProb <- matrix(0, nrow=1,ncol=24)

for (j in 2:24) {
  for (i in 1:90)  { 
    if (SpikesInPeriod[i,j-1] < 0) {
      if (SpikesInPeriod[i,j] < 0) {
        SpikesConditionalOnSpike[j] <- SpikesConditionalOnSpike[j] + 1
      }
      if (SpikesInPeriod[i,j] == 0) {
        NoSpikesConditionalOnSpike[j] <- NoSpikesConditionalOnSpike[j] +1 
      }
    }
    if (SpikesInPeriod[i,j-1] == 0) {
      if (SpikesInPeriod[i,j] < 0) {
        SpikesConditionalOnNoSpike[j] <- SpikesConditionalOnNoSpike[j] + 1
      }
      if (SpikesInPeriod[i,j] == 0) {
        NoSpikesConditionalOnNoSpike[j] <- NoSpikesConditionalOnNoSpike[j] +1 
      }
    }
  }
  if (SpikesConditionalOnSpike[j]+NoSpikesConditionalOnSpike[j] == 0) {
    SpikesConditionalOnSpikeProb[j] = 0
  }
  if (SpikesConditionalOnSpike[j]+NoSpikesConditionalOnSpike[j] < 0) {
    SpikesConditionalOnSpikeProb[j] <- SpikesConditionalOnSpike[j]/(SpikesConditionalOnSpike[j]+NoSpikesConditionalOnSpike[j])
  }
  SpikesConditionalOnNoSpikeProb[j] <- SpikesConditionalOnNoSpike[j]/(SpikesConditionalOnNoSpike[j]+ NoSpikesConditionalOnNoSpike[j])
}

#Sample matrix with spikes (for et scenario) - function
CreateRandomErrorMatrix <- function() {
  RandomErrors <- matrix(0, nrow=7, ncol=24)
  for (i in 1:7) {
    for (j in 2:24) {
      SpikeValue <- c(sample(SpikeValueVectorNegative,1, replace=TRUE),0) 
      if (RandomErrors[i,(j-1)] < 0) {
        RandomErrors[i,j] <- sample(SpikeValue,1,replace=FALSE,prob=c(SpikesConditionalOnSpikeProb[j],1-SpikesConditionalOnSpikeProb[j]))
      }
      if (RandomErrors[i,(j-1)] == 0) {
        RandomErrors[i,j] <- sample(SpikeValue,1,replace=FALSE,prob=c(SpikesConditionalOnNoSpikeProb[j],1-SpikesConditionalOnNoSpikeProb[j]))
      }
    }
  }
  return(RandomErrors)
}

DAScenarioHourOneWSpikes <- matrix(0,nrow=7,ncol=SelectedScenarios)
DAScenarioHourTwoWSpikes <- matrix(0,nrow=7,ncol=SelectedScenarios)
DAScenarioHourThreeWSpikes <- matrix(0,nrow=7,ncol=SelectedScenarios)
DAScenarioHourFourWSpikes <- matrix(0,nrow=7,ncol=SelectedScenarios)
DAScenarioHourFiveWSpikes <- matrix(0,nrow=7,ncol=SelectedScenarios)
DAScenarioHourSixWSpikes <- matrix(0,nrow=7,ncol=SelectedScenarios)
DAScenarioHourSevenWSpikes <- matrix(0,nrow=7,ncol=SelectedScenarios)
DAScenarioHourEightWSpikes <- matrix(0,nrow=7,ncol=SelectedScenarios)
DAScenarioHourNineWSpikes <- matrix(0,nrow=7,ncol=SelectedScenarios)
DAScenarioHourTenWSpikes <- matrix(0,nrow=7,ncol=SelectedScenarios)
DAScenarioHourEllevenWSpikes <- matrix(0,nrow=7,ncol=SelectedScenarios)
DAScenarioHourTwelveWSpikes <- matrix(0,nrow=7,ncol=SelectedScenarios)
DAScenarioHourThirteenWSpikes <- matrix(0,nrow=7,ncol=SelectedScenarios)
DAScenarioHourFourteenWSpikes <- matrix(0,nrow=7,ncol=SelectedScenarios)
DAScenarioHourFifteenWSpikes <- matrix(0,nrow=7,ncol=SelectedScenarios)
DAScenarioHourSixteenWSpikes <- matrix(0,nrow=7,ncol=SelectedScenarios)
DAScenarioHourSeventeenWSpikes <- matrix(0,nrow=7,ncol=SelectedScenarios)
DAScenarioHourEighteenWSpikes <- matrix(0,nrow=7,ncol=SelectedScenarios)
DAScenarioHourNineteenWSpikes <- matrix(0,nrow=7,ncol=SelectedScenarios)
DAScenarioHourTwentyWSpikes <- matrix(0,nrow=7,ncol=SelectedScenarios)
DAScenarioHourTwentyOneWSpikes <- matrix(0,nrow=7,ncol=SelectedScenarios)
DAScenarioHourTwentyTwoWSpikes <- matrix(0,nrow=7,ncol=SelectedScenarios)
DAScenarioHourTwentyThreeWSpikes <- matrix(0,nrow=7,ncol=SelectedScenarios)
DAScenarioHourTwentyFourWSpikes <- matrix(0,nrow=7,ncol=SelectedScenarios)

#create empty 
for (i in 1:SelectedScenarios) { 
  REmatrix <- CreateRandomErrorMatrix()
  DAScenarioHourOneWSpikes[,i] <- DAScenarioHourOneBX[,i]+ REmatrix[,1]+ predict(FitBefore$V1,data.frame(x=c(1209:1215)))
  DAScenarioHourTwoWSpikes[,i] <- DAScenarioHourTwoBX[,i]+ REmatrix[,2] + predict(FitBefore$V2,data.frame(x=c(1209:1215)))
  DAScenarioHourThreeWSpikes[,i] <- DAScenarioHourThreeBX[,i]+ REmatrix[,3] + predict(FitBefore$V3,data.frame(x=c(1209:1215)))
  DAScenarioHourFourWSpikes[,i] <- DAScenarioHourFourBX[,i]+ REmatrix[,4] + predict(FitBefore$V4,data.frame(x=c(1209:1215)))
  DAScenarioHourFiveWSpikes[,i] <- DAScenarioHourFiveBX[,i]+ REmatrix[,5] + predict(FitBefore$V5,data.frame(x=c(1209:1215)))
  DAScenarioHourSixWSpikes[,i] <- DAScenarioHourSixBX[,i]+ REmatrix[,6] + predict(FitBefore$V6,data.frame(x=c(1209:1215)))
  DAScenarioHourSevenWSpikes[,i] <- DAScenarioHourSevenBX[,i]+ REmatrix[,7]+ predict(FitBefore$V7,data.frame(x=c(1209:1215)))
  DAScenarioHourEightWSpikes[,i] <- DAScenarioHourEightBX[,i]+ REmatrix[,8] + predict(FitBefore$V8,data.frame(x=c(1209:1215)))
  DAScenarioHourNineWSpikes[,i] <- DAScenarioHourNineBX[,i]+ REmatrix[,9] + predict(FitBefore$V9,data.frame(x=c(1209:1215)))
  DAScenarioHourTenWSpikes[,i] <- DAScenarioHourTenBX[,i]+ REmatrix[,10] + predict(FitBefore$V10,data.frame(x=c(1209:1215)))
  DAScenarioHourEllevenWSpikes[,i] <- DAScenarioHourEllevenBX[,i]+ REmatrix[,11] + predict(FitBefore$V11,data.frame(x=c(1209:1215)))
  DAScenarioHourTwelveWSpikes[,i] <- DAScenarioHourTwelveBX[,i]+ REmatrix[,12] + predict(FitBefore$V12,data.frame(x=c(1209:1215)))
  DAScenarioHourThirteenWSpikes[,i] <- DAScenarioHourThirteenBX[,i]+ REmatrix[,13]+ predict(FitBefore$V13,data.frame(x=c(1209:1215)))
  DAScenarioHourFourteenWSpikes[,i] <- DAScenarioHourFourteenBX[,i]+ REmatrix[,14] + predict(FitBefore$V14,data.frame(x=c(1209:1215)))
  DAScenarioHourFifteenWSpikes[,i] <- DAScenarioHourFifteenBX[,i]+ REmatrix[,15]+ predict(FitBefore$V15,data.frame(x=c(1209:1215)))
  DAScenarioHourSixteenWSpikes[,i] <- DAScenarioHourSixteenBX[,i]+ REmatrix[,16] + predict(FitBefore$V16,data.frame(x=c(1209:1215)))
  DAScenarioHourSeventeenWSpikes[,i] <- DAScenarioHourSeventeenBX[,i]+ REmatrix[,17] + predict(FitBefore$V17,data.frame(x=c(1209:1215)))
  DAScenarioHourEighteenWSpikes[,i] <- DAScenarioHourEighteenBX[,i]+ REmatrix[,18] + predict(FitBefore$V18,data.frame(x=c(1209:1215)))
  DAScenarioHourNineteenWSpikes[,i] <- DAScenarioHourNineteenBX[,i]+ REmatrix[,19]+ predict(FitBefore$V19,data.frame(x=c(1209:1215)))
  DAScenarioHourTwentyWSpikes[,i] <- DAScenarioHourTwentyBX[,i]+ REmatrix[,20] + predict(FitBefore$V20,data.frame(x=c(1209:1215)))
  DAScenarioHourTwentyOneWSpikes[,i] <- DAScenarioHourTwentyOneBX[,i]+ REmatrix[,21] + predict(FitBefore$V21,data.frame(x=c(1209:1215)))
  DAScenarioHourTwentyTwoWSpikes[,i] <- DAScenarioHourTwentyTwoBX[,i]+ REmatrix[,22] + predict(FitBefore$V22,data.frame(x=c(1209:1215)))
  DAScenarioHourTwentyThreeWSpikes[,i] <- DAScenarioHourTwentyThreeBX[,i]+ REmatrix[,23] + predict(FitBefore$V23,data.frame(x=c(1209:1215)))
  DAScenarioHourTwentyFourWSpikes[,i] <- DAScenarioHourTwentyFourBX[,i] + REmatrix[,24] + predict(FitBefore$V24,data.frame(x=c(1209:1215)))
}

DAMatrixOne <- cbind(DAScenarioHourOneWSpikes,DAScenarioHourTwoWSpikes,DAScenarioHourThreeWSpikes,DAScenarioHourFourWSpikes,DAScenarioHourFiveWSpikes, DAScenarioHourSixWSpikes, DAScenarioHourSevenWSpikes, DAScenarioHourEightWSpikes, DAScenarioHourNineWSpikes, DAScenarioHourTenWSpikes, DAScenarioHourEllevenWSpikes, DAScenarioHourTwelveWSpikes)
DAMatrixTwo <- cbind(DAScenarioHourThirteenWSpikes,DAScenarioHourFourteenWSpikes,DAScenarioHourFifteenWSpikes,DAScenarioHourSixteenWSpikes,DAScenarioHourSeventeenWSpikes, DAScenarioHourEighteenWSpikes, DAScenarioHourNineteenWSpikes, DAScenarioHourTwentyWSpikes, DAScenarioHourTwentyOneWSpikes, DAScenarioHourTwentyTwoWSpikes, DAScenarioHourTwentyThreeWSpikes,DAScenarioHourTwentyFourWSpikes)
DAMatrix <- cbind(DAMatrixOne, DAMatrixTwo)

#------------- PUT the weekly scenarios in weekly matrix --------------- #
WeeklyScenarios <- matrix(0,nrow=168,ncol=SelectedScenarios)

for (i in 1:SelectedScenarios) {  
  NY <- matrix(0,nrow=7, ncol=24)
  for (j in 1:24) {
    NY[,j] <- DAMatrix[,i+(j-1)*SelectedScenarios]
  }
  
  for (k in 1:7) {
    for (j in 1:24) {
      WeeklyScenarios[j+24*(k-1),i] <- NY[k,j]
    }
  }
} 


#2016 verdier - uke 17 
Verdier2016<- read.xls("RealValues_2304-2904.xls", header=FALSE)
Verdier2016ts <- ts(Verdier2016,start=c(2013,1),frequency=365)

WeeklyScenariosIaar <- matrix(0,nrow=168,ncol=1)
for (i in 1:7) {
  for (j in 1:24) {
    WeeklyScenariosIaar[j+24*(i-1),1] <- Verdier2016[j,i]
  }
}


############# PLOT RESULTS DATA #############

a <- 1.6
Xlimit <- c(1:1208)


#Plot average scenario and historical
SD25 <- matrix(0,nrow=1,ncol=168)
SD75 <- matrix(0,nrow=1,ncol=168)
MAX <- matrix(0,nrow=1,ncol=168)
MIN <- matrix(0,nrow=1,ncol=168)

for (i in 1:168) {
  m <- mean(WeeklyScenarios[i,])
  SD25[1,i] <- m - sd(WeeklyScenarios[i,])
  SD75[1,i] <- m + sd(WeeklyScenarios[i,])
  MAX[1,i] <- max(WeeklyScenarios[i,])
  MIN[1,i] <- min(WeeklyScenarios[i,])
}

d <- 1.7

#Average scenario
par(mfcol=c(3,1),mar=c(0, 3.1, 1.8, 0.5), oma=c(2.5,0.5,0.5,0.5)) #mar = inner margin, oma=outer margin - it goes c(bottom, left, top, right) 
c <- rainbow(SelectedScenarios)
plot(AverageWeeklyScenarios[,1], type="l",  main="", col="darkgreen", xlab="Days", ylab="NOK/MWh", ylim=c(0,570), xlim=c(10,168), yaxs="i", xaxs ="i", axes=F, cex.axis=0.8, cex.main = d, mgp=c(1.7, .3, 0), cex.lab=a)
#Scenario i år 
par(new=TRUE)
plot(WeeklyScenariosIaar[,1], type="l", col="blue", xlab="", ylab="", xaxt='n', yaxt='n', yaxs="i", xaxs ="i", bty="n", ylim=c(0,570), xlim=c(10,168))
#MAX/MIN
par(new=TRUE)
plot(MAX[1,],type="l", lty=2, col="red", xlab="", ylab="", xaxt='n', yaxt='n', yaxs="i", xaxs ="i", bty="n", ylim=c(0,570), xlim=c(10,168))
par(new=TRUE)
plot(MIN[1,], type="l", lty=2, col="red", xlab="", ylab="", xaxt='n', yaxt='n', yaxs="i", xaxs ="i", bty="n", ylim=c(0,570), xlim=c(10,168))
#SD25 og 75 
par(new=TRUE)
plot(SD25[1,], type="l", lty=2, col="black", xlab="", ylab="", xaxt='n', yaxt='n', yaxs="i", xaxs ="i", bty="n", ylim=c(0,570), xlim=c(10,168))
par(new=TRUE)
plot(SD75[1,], type="l", lty=2, col="black", xlab="", ylab="", xaxt='n', yaxt='n', yaxs="i", xaxs ="i", bty="n", ylim=c(0,570), xlim=c(10,168))
legend(76, 620,y.intersp=0.38, legend=c("Min/max","1st/3rd quartile","Mean","Realized prices"), col=c("red","black","darkgreen","blue"), lty=1, cex=a, box.lty=0,text.width=25, seg.len=0.7, bty ="n", ncol=2) 
#axis(1, cex.axis=0.8,  mgp=c(1.35, .3, 0));
title("Day-ahead scenarios", line = 0.3, cex.main=d)
axis(2, cex.axis=a, mgp=c(3, .5, 0));box()



#Find scenarios for the primary reserve prices 
sd25 <- matrix(0,nrow=1,ncol=6)
sd75 <- matrix(0,nrow=1,ncol=6)
max <- matrix(0,nrow=1,ncol=6)
min <- matrix(0,nrow=1,ncol=6)
mean <- matrix(0,nrow=1,ncol=6)

for (i in 1:6) {
  MEAN_prim[1,i] <- mean(PrimCorrectScenarios[,i])
  sd25[1,i] <- MEAN_prim[1,i] - sd(PrimCorrectScenarios[,i])
  sd75[1,i] <- MEAN_prim[1,i] + sd(PrimCorrectScenarios[,i])
  max[1,i] <- max(PrimCorrectScenarios[,i])
  min[1,i] <- min(PrimCorrectScenarios[,i])
}

RealizedNightWeekend <- 35
RealizedDayWeekend <- 45
RealizedEveningWeekend <- 45
RealizedNightWeek <- 45
RealizedDayWeek <- 20
RealizedEveningWeek <- 35



#Add these values after each other in correct order 
PRIMPRICES <- marix(0, nrow=5, ncol=168)

Weekend_mean <- cbind(matrix(MEAN_prim[1,4],nrow=1,ncol=8),matrix(MEAN_prim[1,5],nrow=1,ncol=12),matrix(MEAN_prim[1,4],nrow=1,ncol=4))
Weekday_mean <- cbind(matrix(MEAN_prim[1,1],nrow=1,ncol=8),matrix(MEAN_prim[1,2],nrow=1,ncol=12),matrix(MEAN_prim[1,3],nrow=1,ncol=4))
Weekly_mean <- cbind(Weekend_mean,Weekend_mean,Weekday_mean,Weekday_mean,Weekday_mean,Weekday_mean,Weekday_mean)

Weekend_sd25 <- cbind(matrix(sd25[1,4],nrow=1,ncol=8),matrix(sd25[1,5],nrow=1,ncol=12),matrix(sd25[1,4],nrow=1,ncol=4))
Weekday_sd25 <- cbind(matrix(sd25[1,1],nrow=1,ncol=8),matrix(sd25[1,2],nrow=1,ncol=12),matrix(sd25[1,3],nrow=1,ncol=4))
Weekly_sd25 <- cbind(Weekend_sd25,Weekend_sd25,Weekday_sd25,Weekday_sd25,Weekday_sd25,Weekday_sd25,Weekday_sd25)

Weekend_sd75 <- cbind(matrix(sd75[1,4],nrow=1,ncol=8),matrix(sd75[1,5],nrow=1,ncol=12),matrix(sd75[1,4],nrow=1,ncol=4))
Weekday_sd75 <- cbind(matrix(sd75[1,1],nrow=1,ncol=8),matrix(sd75[1,2],nrow=1,ncol=12),matrix(sd75[1,3],nrow=1,ncol=4))
Weekly_sd75 <- cbind(Weekend_sd75,Weekend_sd75,Weekday_sd75,Weekday_sd75,Weekday_sd75,Weekday_sd75,Weekday_sd75)

Weekend_real <- cbind(matrix(RealizedNightWeekend,nrow=1,ncol=8),matrix(RealizedDayWeekend,nrow=1,ncol=12),matrix(RealizedEveningWeekend,nrow=1,ncol=4))
Weekday_real <- cbind(matrix(RealizedNightWeek,nrow=1,ncol=8),matrix(RealizedDayWeek,nrow=1,ncol=12),matrix(RealizedEveningWeek,nrow=1,ncol=4))
Weekly_real <- cbind(Weekend_real,Weekend_real,Weekday_real,Weekday_real,Weekday_real,Weekday_real,Weekday_real)

Weekend_min <- cbind(matrix(min[1,4],nrow=1,ncol=8),matrix(min[1,5],nrow=1,ncol=12),matrix(min[1,4],nrow=1,ncol=4))
Weekday_min <- cbind(matrix(min[1,1],nrow=1,ncol=8),matrix(min[1,2],nrow=1,ncol=12),matrix(min[1,3],nrow=1,ncol=4))
Weekly_min <- cbind(Weekend_min,Weekend_min,Weekday_min,Weekday_min,Weekday_min,Weekday_min,Weekday_min)

Weekend_max<- cbind(matrix(max[1,4],nrow=1,ncol=8),matrix(max[1,5],nrow=1,ncol=12),matrix(max[1,4],nrow=1,ncol=4))
Weekday_max<- cbind(matrix(max[1,1],nrow=1,ncol=8),matrix(max[1,2],nrow=1,ncol=12),matrix(max[1,3],nrow=1,ncol=4))
Weekly_max <- cbind(Weekend_max,Weekend_max,Weekday_max,Weekday_max,Weekday_max,Weekday_max,Weekday_max)


#par(mfcol=c(1,1),mar=c(0.5, 2.5, 1, 0.5), oma=c(2,0.5,0.5,0.5)) #mar = inner margin, oma=outer margin - it goes c(bottom, left, top, right) 
c <- rainbow(SelectedScenarios)
plot(Weekly_mean[1,], type="l",col="darkgreen", main="", xlab="Days", ylab="NOK/MWh", ylim=c(-50,570), xlim=c(0,168), yaxs="i", xaxs ="i", axes=F, cex.axis=0.8,  cex.main = d, mgp=c(1.7, .3, 0), cex.lab=a)
#Scenario i år 
par(new=TRUE)
plot(Weekly_real[1,], type="l", col="purple", xlab="", ylab="", xaxt='n', yaxt='n', yaxs="i", xaxs ="i", bty="n", ylim=c(-50,570), xlim=c(0,168))
#MAX/MIN
par(new=TRUE)
plot(Weekly_max[1,],type="l", lty=2, col="red", xlab="", ylab="", xaxt='n', yaxt='n', yaxs="i", xaxs ="i", bty="n", ylim=c(-50,570), xlim=c(0,168))
par(new=TRUE)
plot(Weekly_min[1,], type="l", lty=2, col="red", xlab="", ylab="", xaxt='n', yaxt='n', yaxs="i", xaxs ="i", bty="n", ylim=c(-50,570), xlim=c(0,168))
#SD25 og 75 
par(new=TRUE)
plot(Weekly_sd25[1,], type="l",lty=2, col="black", xlab="", ylab="", xaxt='n', yaxt='n', yaxs="i", xaxs ="i", bty="n", ylim=c(-50,570), xlim=c(0,168))
par(new=TRUE)
plot(Weekly_sd75[1,], type="l", lty=2, col="black", xlab="", ylab="", xaxt='n', yaxt='n', yaxs="i", xaxs ="i", bty="n", ylim=c(-50,570), xlim=c(0,168))
legend(70, 620,y.intersp=0.38, legend=c("Min/max","1st/3rd quartile","Mean","Realized prices"), col=c("red","black","darkgreen","purple"), lty=1, cex=a, box.lty=0,text.width=25, seg.len=0.7, bty ="n", ncol=2) 
#axis(1, cex.axis=0.8,  mgp=c(1.35, .3, 0));
title("Primary reserve scenarios", line = 0.3, cex.main=d)
axis(2, cex.axis=a, mgp=c(3, .5, 0));box()


#PLOT DA OG PRIMPRISER I SAMME GRAF
#par(mfcol=c(1,1),mar=c(0.5, 2.5, 1, 0.5), oma=c(2,0.5,0.5,0.5)) #mar = inner margin, oma=outer margin - it goes c(bottom, left, top, right) 
c <- rainbow(SelectedScenarios)
plot(AverageWeeklyScenarios[,1], type="l", lty=2, main="", col="blue", xlab="Days", ylab="NOK/MWh", ylim=c(0,570), xlim=c(0,168), yaxs="i", xaxs ="i", axes=F, cex.axis=0.8,  cex.main = d, mgp=c(1.7, .3, 0), cex.lab=a)
#Scenario i år 
par(new=TRUE)
plot(WeeklyScenariosIaar[,1], type="l", col="blue", xlab="", ylab="", xaxt='n', yaxt='n', yaxs="i", xaxs ="i", bty="n", ylim=c(0,570), xlim=c(0,168))
#MAX/MIN
par(new=TRUE)
plot(Weekly_mean[1,],type="l", lty=2,col="purple", xlab="", ylab="", xaxt='n', yaxt='n', yaxs="i", xaxs ="i", bty="n", ylim=c(0,570), xlim=c(0,168))
#par(new=TRUE)
#plot(Weekly_min[1,], type="l", lty=2, col="red", xlab="", ylab="", xaxt='n', yaxt='n', yaxs="i", xaxs ="i", bty="n", ylim=c(-50,400), xlim=c(0,168))
#SD25 og 75 
#par(new=TRUE)
#plot(Weekly_sd25[1,], type="l", col="black", xlab="", ylab="", xaxt='n', yaxt='n', yaxs="i", xaxs ="i", bty="n", ylim=c(-50,400), xlim=c(0,168))
par(new=TRUE)
plot(Weekly_real[1,], type="l", col="purple", xlab="", ylab="", xaxt='n', yaxt='n', yaxs="i", xaxs ="i", bty="n", ylim=c(0,570), xlim=c(0,168))
legend(70, 620,y.intersp=0.38, legend=c("","","DA prices","Primary reserve prices"), col=c("white","white","blue","purple"), lty=1, cex=a, box.lty=0,text.width=25, seg.len=0.7, bty ="n", ncol=2) 
title(xlab = "Hours", outer = TRUE, line = 1.5, cex.lab=a)
title("Day-ahead and primary reserve scenarios", line = 0.3, cex.main=d)
axis(1, cex.axis=a,  mgp=c(1.7, .7, 0));axis(2, cex.axis=a, mgp=c(1.5, .45, 0));box()



